@@ -22,7 +22,7 @@ from hod_models import HOD_models
 from bias import Bias as bias
 from fort.routines import hod_routines as fort
 
-#TODO: make hankel transform better -- its pretty wobbly.
+# TODO: make hankel transform better -- its pretty wobbly.
 #===============================================================================
 # The class itself
 #===============================================================================
@@ -70,7 +70,7 @@ class HOD(object):
                  central=False, nonlinear=False, scale_dependent_bias=True,
                  halo_exclusion=None, ** pert_kwargs):
 
-        #A dictionary of all HOD parameters
+        # A dictionary of all HOD parameters
         self.hodmod_params = {"M_1":M_1,
                               'alpha': alpha,
                               "M_min":M_min,
@@ -87,9 +87,9 @@ class HOD(object):
                                a, [self.hodmod_params, pert_kwargs], {})
 
 
-        #Initially save parameters to the class.
-        #We must do this because update() checks if values are different,
-        #which doesn't work if there are no values there to begin with
+        # Initially save parameters to the class.
+        # We must do this because update() checks if values are different,
+        # which doesn't work if there are no values there to begin with
         self.halo_profile = halo_profile
         self.cm_relation = cm_relation
         self.bias_model = bias_model
@@ -114,26 +114,26 @@ class HOD(object):
         basically just deletes everything. We only keep properties that 
         may be used more than once or called more than once.
         """
-        #We use a somewhat tiered approach here.
-        #We first update all simple HOD parameters, then whatever is left
-        #is sent to Perturbations(), which will weed out stupid values.
+        # We use a somewhat tiered approach here.
+        # We first update all simple HOD parameters, then whatever is left
+        # is sent to Perturbations(), which will weed out stupid values.
 
-        startfresh = False  #Get's set to true if anything is actually updated
+        startfresh = False  # Get's set to true if anything is actually updated
 
-        #First do hod_model parameters - update them and delete entries from kwargs.
+        # First do hod_model parameters - update them and delete entries from kwargs.
         hodmod_p = {k:v for k, v in kwargs.iteritems() if k in self.hodmod_params}
         if hodmod_p:
             self.hodmod_params.update(hodmod_p)
 
-            #Delete the entries we've used from kwargs
+            # Delete the entries we've used from kwargs
             for k in hodmod_p:
                 del kwargs[k]
 
-            #Now actually update the hodmod class
+            # Now actually update the hodmod class
             self._hodmod = HOD_models(**self.hodmod_params)
             startfresh = True
 
-        #Now go through the rest of the keys and set values
+        # Now go through the rest of the keys and set values
         for k in kwargs.keys():
             if k == 'halo_profile':
                 new = kwargs.pop(k)
@@ -177,9 +177,9 @@ class HOD(object):
                     self.scale_dependent_bias = new
                     startfresh = True
 
-        #All that's left should be Perturbations() args
-        #However, we don't want to call a new class each time,
-        #we usually want to update.
+        # All that's left should be Perturbations() args
+        # However, we don't want to call a new class each time,
+        # we usually want to update.
         if kwargs:
             try:
                 self.pert.update(cut_fit=False, **kwargs)
@@ -188,8 +188,8 @@ class HOD(object):
 
             startfresh = True
 
-        ##Now that we've updated the base parameters, we will need to
-        ##re-calculate basically everything.
+        # #Now that we've updated the base parameters, we will need to
+        # #re-calculate basically everything.
         # We only retain (and therefore must delete here) properties that are
         # either used more than once in the class, OR that we expect may want
         # to be accessed by the user and may cost a bit.
@@ -319,7 +319,7 @@ class HOD(object):
         try:
             return self.__profile
         except:
-            self.__profile = profiles(self.pert.cosmo_params['mean_dens'],
+            self.__profile = profiles(self.pert.cosmo_params.mean_dens,
                                       self.pert.delta_halo,
                                       profile=self.halo_profile,
                                       cm_relation=self.cm_relation)
@@ -366,7 +366,7 @@ class HOD(object):
         try:
             return self.__mean_gal_den
         except:
-            #Integrand is just the density of galaxies at mass M
+            # Integrand is just the density of galaxies at mass M
             integrand = self.pert.dndm * self.n_tot
 
             self.__mean_gal_den = intg.simps(integrand, self.pert.M)
@@ -383,11 +383,12 @@ class HOD(object):
             return self.__matter_power
         except:
             if self.nonlinear:
-                lnk, self.__matter_power = tools.non_linear_power(self.pert.lnkh,
-                                                                  **self.pert.camb_params)
-                #Now we need to normalise
-                #FIXME: get index of self.lnkh which is above -5.3
-                ind = np.where(self.pert.lnkh > -5.3)[0][0]
+                lnk, self.__matter_power = tools.non_linear_power(self.pert.lnk,
+                                                                  **dict(self.pert.cosmo_params.pycamb_dict(),
+                                                                       **self.pert._camb_options))
+                # Now we need to normalise
+                # FIXME: get index of self.lnkh which is above -5.3
+                ind = np.where(self.pert.lnk > -5.3)[0][0]
                 self.__matter_power += self.pert.power[ind] - self.__matter_power[0]
                 self.__matter_power = np.concatenate((self.pert.power[:ind], self.__matter_power))
             else:
@@ -402,9 +403,9 @@ class HOD(object):
         try:
             return self.__dm_corr
         except:
-            self.__dm_corr = fort.power_to_corr(nlnk=len(self.pert.lnkh),
+            self.__dm_corr = fort.power_to_corr(nlnk=len(self.pert.lnk),
                                                 nr=len(self.r),
-                                                lnk=self.pert.lnkh,
+                                                lnk=self.pert.lnk,
                                                 r=self.r,
                                                 power=np.exp(self.matter_power))
             return self.__dm_corr
@@ -414,7 +415,7 @@ class HOD(object):
         """
         The sat-sat part of the 1-halo term of the galaxy power spectrum
         """
-        u = self.profile.u(np.exp(self.pert.lnkh), self.pert.M, self.pert.z)
+        u = self.profile.u(np.exp(self.pert.lnk), self.pert.M, self.pert.z)
         result = fort.power_gal_1h_ss(nlnk=len(self.pert.lnk),
                                       nm=len(self.pert.M),
                                       u=np.asfortranarray(u),
@@ -432,7 +433,7 @@ class HOD(object):
         """The 1-halo galaxy correlation for sat-sat pairs"""
         result = fort.power_to_corr(nlnk=len(self.pert.lnk),
                                     nr=len(self.r),
-                                    lnk=self.pert.lnkh,
+                                    lnk=self.pert.lnk,
                                     r=self.r,
                                     power=self._power_gal_1h_ss)
         return result
@@ -449,7 +450,7 @@ class HOD(object):
                                      ncen=self.n_cen,
                                      nsat=self.n_sat,
                                      rho=np.asfortranarray(rho),
-                                     mean_dens=self.pert.cosmo_params['mean_dens'],
+                                     mean_dens=self.pert.cosmo_params.mean_dens,
                                      delta_halo=self.pert.delta_halo)
         result /= self.mean_gal_den ** 2
         return result
@@ -474,7 +475,7 @@ class HOD(object):
                                                       rho=np.asfortranarray(rho),
                                                       lam=np.asfortranarray(lam),
                                                       central=self.hodmod_params['central'],
-                                                      mean_dens=self.pert.cosmo_params['mean_dens'],
+                                                      mean_dens=self.pert.cosmo_params.mean_dens,
                                                       delta_halo=self.pert.delta_halo)
                 self.__corr_gal_1h /= self.mean_gal_den ** 2
 
@@ -486,7 +487,7 @@ class HOD(object):
     def power_gal_2h(self, r_index=None):
         """The 2-halo term of the galaxy power spectrum - NOT LOGGED"""
         if not self.scale_dependent_bias and self.halo_exclusion in [None, "schneider"]:
-            u = self.profile.u(np.exp(self.pert.lnkh), self.pert.M, self.pert.z)
+            u = self.profile.u(np.exp(self.pert.lnk), self.pert.M, self.pert.z)
             pg2h = fort.power_gal_2h(nlnk=len(self.pert.lnk),
                                      nm=len(self.pert.M),
                                      u=np.asfortranarray(u),
@@ -496,7 +497,7 @@ class HOD(object):
                                      dndm=self.pert.dndm,
                                      mass=self.pert.M)
 
-        else:  #We must use a value of r to get pg2h
+        else:  # We must use a value of r to get pg2h
             pg2h = np.zeros_like(self.pert.lnk)
 
             if self.scale_dependent_bias:
@@ -528,7 +529,7 @@ class HOD(object):
                                          np.log(integrand_m[integrand_m > 0]), k=1)
                     def integ(m1, m2, r):
                         rv1, rv2 = tools.virial_radius(np.exp(np.array([m1, m2])),
-                                                      self.pert.cosmo_params['mean_dens'],
+                                                      self.pert.cosmo_params.mean_dens,
                                                       self.pert.delta_halo)
 
                         return np.exp(m1 + m2) * np.exp(integrand_ms(m1) + integrand_ms(m2)) * tools.overlapping_halo_prob(r, rv1, rv2)
@@ -556,7 +557,7 @@ class HOD(object):
 
                     def integ(m1, m2, r):
                         rv1, rv2 = tools.virial_radius(np.exp(np.array([m1, m2])),
-                                                      self.pert.cosmo_params['mean_dens'],
+                                                      self.pert.cosmo_params.mean_dens,
                                                       self.pert.delta_halo)
 
                         return np.exp(m1 + m2) * np.exp(integrand(m1) + integrand(m2)) * tools.overlapping_halo_prob(r, rv1, rv2)
@@ -572,7 +573,7 @@ class HOD(object):
         pg2h = np.exp(self.matter_power) * pg2h ** 2 / self.mean_gal_den ** 2
 
         if self.halo_exclusion == 'schneider':
-            #should be k,r/h, but we use k/h and r (which gives the same)
+            # should be k,r/h, but we use k/h and r (which gives the same)
             pg2h *= tools.exclusion_window(np.exp(self.pert.lnk), r=2)
         self.post_pg2h = pg2h
         return pg2h
@@ -586,7 +587,7 @@ class HOD(object):
             if not self.scale_dependent_bias and self.halo_exclusion in [None, 'schneider']:
                 self.__corr_gal_2h = fort.power_to_corr(nlnk=len(self.pert.lnk),
                                                        nr=len(self.r),
-                                                       lnk=self.pert.lnkh,
+                                                       lnk=self.pert.lnk,
                                                        r=self.r,
                                                        power=self.power_gal_2h())
             else:
@@ -596,7 +597,7 @@ class HOD(object):
 
                     self.__corr_gal_2h[i] = fort.power_to_corr(nlnk=len(self.pert.lnk),
                                                                nr=len(self.r),
-                                                               lnk=self.pert.lnkh,
+                                                               lnk=self.pert.lnk,
                                                                r=self.r,
                                                                power=power)[0]
             return self.__corr_gal_2h
